import datetime
import time
import subprocess
from concurrent.futures import ThreadPoolExecutor, as_completed


thread_num = 2


def execCmd(cmd,logger):
    try_num = 0
    command = ' '.join(cmd)
    while True:
        logger.info('[{}]开始第{}执行命令：{}'.format(datetime.datetime.now(), try_num, command))
        process = subprocess.Popen(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            universal_newlines=True
        )

        flag=0
        while True:
            output = process.stdout.readline()
            if output == '' and process.poll() is not None:
                break
            if output and flag % 100==0:
                logger.info(output.strip())
            flag+=1

        returncode = process.poll()
        logger.info("[{}]第{}执行命令{}返回码：{}".format(datetime.datetime.now(), try_num, command, returncode))
        if returncode == 0:
            break
        time.sleep(1)
        try_num += 1


def load_model(save_path,target_url,file_list,logger):
    """
    Args:
        save_path:模型保存路径，例如：/data/algorithm/models/bge-base-zh-v1.5
        target_url:下载url，例如：https://inwise.macrowing.com/models/glm/glm3
        file_list:target_url下面的需要下载的文件列表，例如：["config.json","configuration_chatglm.py","modeling_chatglm.py",
            "pytorch_model.bin","quantization.py","tokenization_chatglm.py","tokenizer_config.json","tokenizer.model"]
        logger:日志对象

    Returns:
    """
    # 获取项目信息
    cmds = []

    # 需要执行的命令列表
    for file in file_list:
        # 获取路径
        url = "{}/{}".format(target_url, file)
        # 断点下载指令
        cmds.append(['wget', '-T', '60', '-c', url, '-P', save_path, '--no-check-certificate'])

    logger.info("程序开始%s" % datetime.datetime.now())
    with ThreadPoolExecutor(max_workers=thread_num) as t:
        all_task = [t.submit(execCmd, cmd,logger) for cmd in cmds]
        finish_num = 0
        for future in as_completed(all_task):
            finish_num += 1
            logger.info('[{}]************已完成：{}/{}********************'.format(datetime.datetime.now(), finish_num,len(all_task)))
    logger.info("程序结束%s" % datetime.datetime.now())



